@@ -6,17 +6,15 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 from .client import InfluxDBClient
-from .client import InfluxDBClusterClient
 from .dataframe_client import DataFrameClient
 from .helper import SeriesHelper
 
 
 __all__ = [
     'InfluxDBClient',
-    'InfluxDBClusterClient',
     'DataFrameClient',
     'SeriesHelper',
 ]
 
 
-__version__ = '3.0.0'
+__version__ = '4.1.1'
