@@ -7,17 +7,13 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-from functools import wraps
 import json
 import socket
-import time
-import threading
-import random
 import requests
 import requests.exceptions
 from sys import version_info
 
-from influxdb.line_protocol import make_lines
+from influxdb.line_protocol import make_lines, quote_ident, quote_literal
 from influxdb.resultset import ResultSet
 from .exceptions import InfluxDBClientError
 from .exceptions import InfluxDBServerError
@@ -57,6 +53,9 @@ class InfluxDBClient(object):
     :param timeout: number of seconds Requests will wait for your client to
         establish a connection, defaults to None
     :type timeout: int
+    :param retries: number of retries your client will try before aborting,
+        defaults to 3. 0 indicates try until success
+    :type retries: int
     :param use_udp: use UDP to connect to InfluxDB, defaults to False
     :type use_udp: bool
     :param udp_port: UDP port to connect to InfluxDB, defaults to 4444
@@ -74,6 +73,7 @@ class InfluxDBClient(object):
                  ssl=False,
                  verify_ssl=False,
                  timeout=None,
+                 retries=3,
                  use_udp=False,
                  udp_port=4444,
                  proxies=None,
@@ -85,6 +85,7 @@ class InfluxDBClient(object):
         self._password = password
         self._database = database
         self._timeout = timeout
+        self._retries = retries
 
         self._verify_ssl = verify_ssl
 
@@ -114,8 +115,6 @@ class InfluxDBClient(object):
             'Accept': 'text/plain'
         }
 
-    # _baseurl, _host and _port are properties to allow InfluxDBClusterClient
-    # to override them with thread-local variables
     @property
     def _baseurl(self):
         return self._get_baseurl()
@@ -137,8 +136,8 @@ class InfluxDBClient(object):
     def _get_port(self):
         return self.__port
 
-    @staticmethod
-    def from_DSN(dsn, **kwargs):
+    @classmethod
+    def from_DSN(cls, dsn, **kwargs):
         """Return an instance of :class:`~.InfluxDBClient` from the provided
         data source name. Supported schemes are "influxdb", "https+influxdb"
         and "udp+influxdb". Parameters for the :class:`~.InfluxDBClient`
@@ -175,7 +174,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         init_args['port'] = port
         init_args.update(kwargs)
 
-        return InfluxDBClient(**init_args)
+        return cls(**init_args)
 
     def switch_database(self, database):
         """Change the client's database.
@@ -211,6 +210,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param expected_response_code: the expected response code of
             the request, defaults to 200
         :type expected_response_code: int
+        :param headers: headers to add to the request
+        :type headers: dict
         :returns: the response from the request
         :rtype: :class:`requests.Response`
         :raises InfluxDBServerError: if the response code is any server error
@@ -229,9 +230,10 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         if isinstance(data, (dict, list)):
             data = json.dumps(data)
 
-        # Try to send the request a maximum of three times. (see #103)
-        # TODO (aviau): Make this configurable.
-        for i in range(0, 3):
+        # Try to send the request more than once by default (see #103)
+        retry = True
+        _try = 0
+        while retry:
             try:
                 response = self._session.request(
                     method=method,
@@ -246,28 +248,34 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                 )
                 break
             except requests.exceptions.ConnectionError as e:
-                if i < 2:
-                    continue
-                else:
-                    raise e
+                _try += 1
+                if self._retries != 0:
+                    retry = _try < self._retries
 
-        if response.status_code >= 500 and response.status_code < 600:
+        else:
+            raise requests.exceptions.ConnectionError
+
+        if 500 <= response.status_code < 600:
             raise InfluxDBServerError(response.content)
         elif response.status_code == expected_response_code:
             return response
         else:
             raise InfluxDBClientError(response.content, response.status_code)
 
-    def write(self, data, params=None, expected_response_code=204):
+    def write(self, data, params=None, expected_response_code=204,
+              protocol='json'):
         """Write data to InfluxDB.
 
         :param data: the data to be written
-        :type data: dict
+        :type data: (if protocol is 'json') dict
+                    (if protocol is 'line') sequence of line protocol strings
         :param params: additional parameters for the request, defaults to None
         :type params: dict
         :param expected_response_code: the expected response code of the write
             operation, defaults to 204
         :type expected_response_code: int
+        :param protocol: protocol of input data, either 'json' or 'line'
+        :type protocol: str
         :returns: True, if the write operation is successful
         :rtype: bool
         """
@@ -280,23 +288,42 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         else:
             precision = None
 
+        if protocol == 'json':
+            data = make_lines(data, precision).encode('utf-8')
+        elif protocol == 'line':
+            data = ('\n'.join(data) + '\n').encode('utf-8')
+
         self.request(
             url="write",
             method='POST',
             params=params,
-            data=make_lines(data, precision).encode('utf-8'),
+            data=data,
             expected_response_code=expected_response_code,
             headers=headers
         )
         return True
 
+    def _read_chunked_response(self, response, raise_errors=True):
+        result_set = {}
+        for line in response.iter_lines():
+            if isinstance(line, bytes):
+                line = line.decode('utf-8')
+            data = json.loads(line)
+            for result in data.get('results', []):
+                for _key in result:
+                    if type(result[_key]) == list:
+                        result_set.setdefault(_key, []).extend(result[_key])
+        return ResultSet(result_set, raise_errors=raise_errors)
+
     def query(self,
               query,
               params=None,
               epoch=None,
               expected_response_code=200,
               database=None,
-              raise_errors=True):
+              raise_errors=True,
+              chunked=False,
+              chunk_size=0):
         """Send a query to InfluxDB.
 
         :param query: the actual query string
@@ -305,6 +332,11 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param params: additional parameters for the request, defaults to {}
         :type params: dict
 
+        :param epoch: response timestamps to be in epoch format either 'h',
+            'm', 's', 'ms', 'u', or 'ns',defaults to `None` which is
+            RFC3339 UTC format with nanosecond precision
+        :type epoch: str
+
         :param expected_response_code: the expected status code of response,
             defaults to 200
         :type expected_response_code: int
@@ -316,6 +348,14 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             returns errors, defaults to True
         :type raise_errors: bool
 
+        :param chunked: Enable to use chunked responses from InfluxDB.
+            With ``chunked`` enabled, one ResultSet is returned per chunk
+            containing all results within that chunk
+        :type chunked: bool
+
+        :param chunk_size: Size of each chunk to tell InfluxDB to use.
+        :type chunk_size: int
+
         :returns: the queried data
         :rtype: :class:`~.ResultSet`
         """
@@ -328,6 +368,11 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         if epoch is not None:
             params['epoch'] = epoch
 
+        if chunked:
+            params['chunked'] = 'true'
+            if chunk_size > 0:
+                params['chunk_size'] = chunk_size
+
         response = self.request(
             url="query",
             method='GET',
@@ -336,6 +381,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             expected_response_code=expected_response_code
         )
 
+        if chunked:
+            return self._read_chunked_response(response)
+
         data = response.json()
 
         results = [
@@ -357,11 +405,15 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                      retention_policy=None,
                      tags=None,
                      batch_size=None,
+                     protocol='json'
                      ):
         """Write to multiple time series names.
 
         :param points: the list of points to be written in the database
         :type points: list of dictionaries, each dictionary represents a point
+        :type points: (if protocol is 'json') list of dicts, where each dict
+                                            represents a point.
+                    (if protocol is 'line') sequence of line protocol strings.
         :param time_precision: Either 's', 'm', 'ms' or 'u', defaults to None
         :type time_precision: str
         :param database: the database to write the points to. Defaults to
@@ -379,6 +431,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             one database to another or when doing a massive write operation,
             defaults to None
         :type batch_size: int
+        :param protocol: Protocol for writing data. Either 'line' or 'json'.
+        :type protocol: str
         :returns: True, if the operation is successful
         :rtype: bool
 
@@ -392,14 +446,14 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                                    time_precision=time_precision,
                                    database=database,
                                    retention_policy=retention_policy,
-                                   tags=tags)
+                                   tags=tags, protocol=protocol)
             return True
         else:
             return self._write_points(points=points,
                                       time_precision=time_precision,
                                       database=database,
                                       retention_policy=retention_policy,
-                                      tags=tags)
+                                      tags=tags, protocol=protocol)
 
     def _batches(self, iterable, size):
         for i in xrange(0, len(iterable), size):
@@ -410,7 +464,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                       time_precision,
                       database,
                       retention_policy,
-                      tags):
+                      tags,
+                      protocol='json'):
         if time_precision not in ['n', 'u', 'ms', 's', 'm', 'h', None]:
             raise ValueError(
                 "Invalid time precision is given. "
@@ -421,12 +476,15 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                 "InfluxDB only supports seconds precision for udp writes"
             )
 
-        data = {
-            'points': points
-        }
+        if protocol == 'json':
+            data = {
+                'points': points
+            }
 
-        if tags is not None:
-            data['tags'] = tags
+            if tags is not None:
+                data['tags'] = tags
+        else:
+            data = points
 
         params = {
             'db': database or self._database
@@ -439,12 +497,13 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             params['rp'] = retention_policy
 
         if self.use_udp:
-            self.send_packet(data)
+            self.send_packet(data, protocol=protocol)
         else:
             self.write(
                 data=data,
                 params=params,
-                expected_response_code=204
+                expected_response_code=204,
+                protocol=protocol
             )
 
         return True
@@ -471,7 +530,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param dbname: the name of the database to create
         :type dbname: str
         """
-        self.query("CREATE DATABASE \"%s\"" % dbname)
+        self.query("CREATE DATABASE {0}".format(quote_ident(dbname)))
 
     def drop_database(self, dbname):
         """Drop a database from InfluxDB.
@@ -479,7 +538,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param dbname: the name of the database to drop
         :type dbname: str
         """
-        self.query("DROP DATABASE \"%s\"" % dbname)
+        self.query("DROP DATABASE {0}".format(quote_ident(dbname)))
 
     def create_retention_policy(self, name, duration, replication,
                                 database=None, default=False):
@@ -503,9 +562,10 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type default: bool
         """
         query_string = \
-            "CREATE RETENTION POLICY \"%s\" ON \"%s\" " \
-            "DURATION %s REPLICATION %s" % \
-            (name, database or self._database, duration, replication)
+            "CREATE RETENTION POLICY {0} ON {1} " \
+            "DURATION {2} REPLICATION {3}".format(
+                quote_ident(name), quote_ident(database or self._database),
+                duration, replication)
 
         if default is True:
             query_string += " DEFAULT"
@@ -530,7 +590,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type duration: str
         :param replication: the new replication of the existing
             retention policy
-        :type replication: str
+        :type replication: int
         :param default: whether or not to set the modified policy as default
         :type default: bool
 
@@ -538,8 +598,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             should be set. Otherwise the operation will fail.
         """
         query_string = (
-            "ALTER RETENTION POLICY \"{0}\" ON \"{1}\""
-        ).format(name, database or self._database)
+            "ALTER RETENTION POLICY {0} ON {1}"
+        ).format(quote_ident(name), quote_ident(database or self._database))
         if duration:
             query_string += " DURATION {0}".format(duration)
         if replication:
@@ -559,8 +619,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type database: str
         """
         query_string = (
-            "DROP RETENTION POLICY \"{0}\" ON \"{1}\""
-        ).format(name, database or self._database)
+            "DROP RETENTION POLICY {0} ON {1}"
+        ).format(quote_ident(name), quote_ident(database or self._database))
         self.query(query_string)
 
     def get_list_retention_policies(self, database=None):
@@ -583,30 +643,18 @@ localhost:8086/databasename', timeout=5, udp_port=159)
               u'name': u'default',
               u'replicaN': 1}]
             """
+
+        if not (database or self._database):
+            raise InfluxDBClientError(
+                "get_list_retention_policies() requires a database as a "
+                "parameter or the client to be using a database")
+
         rsp = self.query(
-            "SHOW RETENTION POLICIES ON \"%s\"" % (database or self._database)
+            "SHOW RETENTION POLICIES ON {0}".format(
+                quote_ident(database or self._database))
         )
         return list(rsp.get_points())
 
-    def get_list_servers(self):
-        """Get the list of servers in InfluxDB cluster.
-
-        :returns: all nodes in InfluxDB cluster
-        :rtype: list of dictionaries
-
-        :Example:
-
-        ::
-
-            >> servers = client.get_list_servers()
-            >> servers
-            [{'cluster_addr': 'server01:8088',
-              'id': 1,
-              'raft': True,
-              'raft-leader': True}]
-        """
-        return list(self.query("SHOW SERVERS").get_points())
-
     def get_list_users(self):
         """Get the list of all users in InfluxDB.
 
@@ -636,8 +684,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             privileges or not
         :type admin: boolean
         """
-        text = "CREATE USER \"{0}\" WITH PASSWORD '{1}'".format(username,
-                                                                password)
+        text = "CREATE USER {0} WITH PASSWORD {1}".format(
+            quote_ident(username), quote_literal(password))
         if admin:
             text += ' WITH ALL PRIVILEGES'
         self.query(text)
@@ -648,7 +696,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param username: the username to drop
         :type username: str
         """
-        text = "DROP USER {0}".format(username)
+        text = "DROP USER {0}".format(quote_ident(username))
         self.query(text)
 
     def set_user_password(self, username, password):
@@ -659,29 +707,31 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param password: the new password for the user
         :type password: str
         """
-        text = "SET PASSWORD FOR {0} = '{1}'".format(username, password)
+        text = "SET PASSWORD FOR {0} = {1}".format(
+            quote_ident(username), quote_literal(password))
         self.query(text)
 
     def delete_series(self, database=None, measurement=None, tags=None):
         """Delete series from a database. Series can be filtered by
         measurement and tags.
 
-        :param measurement: Delete all series from a measurement
-        :type id: string
-        :param tags: Delete all series that match given tags
-        :type id: dict
         :param database: the database from which the series should be
             deleted, defaults to client's current database
         :type database: str
+        :param measurement: Delete all series from a measurement
+        :type measurement: str
+        :param tags: Delete all series that match given tags
+        :type tags: dict
         """
         database = database or self._database
         query_str = 'DROP SERIES'
         if measurement:
-            query_str += ' FROM "{0}"'.format(measurement)
+            query_str += ' FROM {0}'.format(quote_ident(measurement))
 
         if tags:
-            query_str += ' WHERE ' + ' and '.join(["{0}='{1}'".format(k, v)
-                                                   for k, v in tags.items()])
+            tag_eq_list = ["{0}={1}".format(quote_ident(k), quote_literal(v))
+                           for k, v in tags.items()]
+            query_str += ' WHERE ' + ' AND '.join(tag_eq_list)
         self.query(query_str, database=database)
 
     def grant_admin_privileges(self, username):
@@ -693,7 +743,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         .. note:: Only a cluster administrator can create/drop databases
             and manage users.
         """
-        text = "GRANT ALL PRIVILEGES TO {0}".format(username)
+        text = "GRANT ALL PRIVILEGES TO {0}".format(quote_ident(username))
         self.query(text)
 
     def revoke_admin_privileges(self, username):
@@ -705,7 +755,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         .. note:: Only a cluster administrator can create/ drop databases
             and manage users.
         """
-        text = "REVOKE ALL PRIVILEGES FROM {0}".format(username)
+        text = "REVOKE ALL PRIVILEGES FROM {0}".format(quote_ident(username))
         self.query(text)
 
     def grant_privilege(self, privilege, database, username):
@@ -720,8 +770,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type username: str
         """
         text = "GRANT {0} ON {1} TO {2}".format(privilege,
-                                                database,
-                                                username)
+                                                quote_ident(database),
+                                                quote_ident(username))
         self.query(text)
 
     def revoke_privilege(self, privilege, database, username):
@@ -736,8 +786,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type username: str
         """
         text = "REVOKE {0} ON {1} FROM {2}".format(privilege,
-                                                   database,
-                                                   username)
+                                                   quote_ident(database),
+                                                   quote_ident(username))
         self.query(text)
 
     def get_list_privileges(self, username):
@@ -759,181 +809,25 @@ localhost:8086/databasename', timeout=5, udp_port=159)
              {u'privilege': u'ALL PRIVILEGES', u'database': u'db2'},
              {u'privilege': u'NO PRIVILEGES', u'database': u'db3'}]
         """
-        text = "SHOW GRANTS FOR {0}".format(username)
+        text = "SHOW GRANTS FOR {0}".format(quote_ident(username))
         return list(self.query(text).get_points())
 
-    def send_packet(self, packet):
+    def send_packet(self, packet, protocol='json'):
         """Send an UDP packet.
 
         :param packet: the packet to be sent
-        :type packet: dict
+        :type packet: (if protocol is 'json') dict
+                      (if protocol is 'line') sequence of line protocol strings
+        :param protocol: protocol of input data, either 'json' or 'line'
+        :type protocol: str
         """
-        data = make_lines(packet).encode('utf-8')
+        if protocol == 'json':
+            data = make_lines(packet).encode('utf-8')
+        elif protocol == 'line':
+            data = ('\n'.join(packet) + '\n').encode('utf-8')
         self.udp_socket.sendto(data, (self._host, self.udp_port))
 
 
-class InfluxDBClusterClient(object):
-    """The :class:`~.InfluxDBClusterClient` is the client for connecting
-    to a cluster of InfluxDB servers. Each query hits different host from the
-    list of hosts.
-
-    :param hosts: all hosts to be included in the cluster, each of which
-        should be in the format (address, port),
-        e.g. [('127.0.0.1', 8086), ('127.0.0.1', 9096)]. Defaults to
-        [('localhost', 8086)]
-    :type hosts: list of tuples
-    :param shuffle: whether the queries should hit servers evenly(randomly),
-        defaults to True
-    :type shuffle: bool
-    :param client_base_class: the base class for the cluster client.
-        This parameter is used to enable the support of different client
-        types. Defaults to :class:`~.InfluxDBClient`
-    :param healing_delay: the delay in seconds, counting from last failure of
-        a server, before re-adding server to the list of working servers.
-        Defaults to 15 minutes (900 seconds)
-    """
-
-    def __init__(self,
-                 hosts=[('localhost', 8086)],
-                 username='root',
-                 password='root',
-                 database=None,
-                 ssl=False,
-                 verify_ssl=False,
-                 timeout=None,
-                 use_udp=False,
-                 udp_port=4444,
-                 shuffle=True,
-                 client_base_class=InfluxDBClient,
-                 healing_delay=900,
-                 ):
-        self.clients = [self]  # Keep it backwards compatible
-        self.hosts = hosts
-        self.bad_hosts = []   # Corresponding server has failures in history
-        self.shuffle = shuffle
-        self.healing_delay = healing_delay
-        self._last_healing = time.time()
-        host, port = self.hosts[0]
-        self._hosts_lock = threading.Lock()
-        self._thread_local = threading.local()
-        self._client = client_base_class(host=host,
-                                         port=port,
-                                         username=username,
-                                         password=password,
-                                         database=database,
-                                         ssl=ssl,
-                                         verify_ssl=verify_ssl,
-                                         timeout=timeout,
-                                         use_udp=use_udp,
-                                         udp_port=udp_port)
-        for method in dir(client_base_class):
-            orig_attr = getattr(client_base_class, method, '')
-            if method.startswith('_') or not callable(orig_attr):
-                continue
-
-            setattr(self, method, self._make_func(orig_attr))
-
-        self._client._get_host = self._get_host
-        self._client._get_port = self._get_port
-        self._client._get_baseurl = self._get_baseurl
-        self._update_client_host(self.hosts[0])
-
-    @staticmethod
-    def from_DSN(dsn, client_base_class=InfluxDBClient,
-                 shuffle=True, **kwargs):
-        """Same as :meth:`~.InfluxDBClient.from_DSN`, but supports
-        multiple servers.
-
-        :param shuffle: whether the queries should hit servers
-            evenly(randomly), defaults to True
-        :type shuffle: bool
-        :param client_base_class: the base class for all clients in the
-            cluster. This parameter is used to enable the support of
-            different client types. Defaults to :class:`~.InfluxDBClient`
-
-        :Example:
-
-        ::
-
-            >> cluster = InfluxDBClusterClient.from_DSN('influxdb://usr:pwd\
-@host1:8086,usr:pwd@host2:8086/db_name', timeout=5)
-            >> type(cluster)
-            <class 'influxdb.client.InfluxDBClusterClient'>
-            >> cluster.hosts
-            [('host1', 8086), ('host2', 8086)]
-            >> cluster._client
-             <influxdb.client.InfluxDBClient at 0x7feb438ec950>]
-        """
-        init_args = parse_dsn(dsn)
-        init_args.update(**kwargs)
-        init_args['shuffle'] = shuffle
-        init_args['client_base_class'] = client_base_class
-        cluster_client = InfluxDBClusterClient(**init_args)
-        return cluster_client
-
-    def _update_client_host(self, host):
-        self._thread_local.host, self._thread_local.port = host
-        self._thread_local.baseurl = "{0}://{1}:{2}".format(
-            self._client._scheme,
-            self._client._host,
-            self._client._port
-        )
-
-    def _get_baseurl(self):
-        return self._thread_local.baseurl
-
-    def _get_host(self):
-        return self._thread_local.host
-
-    def _get_port(self):
-        return self._thread_local.port
-
-    def _make_func(self, orig_func):
-
-        @wraps(orig_func)
-        def func(*args, **kwargs):
-            now = time.time()
-            with self._hosts_lock:
-                if (self.bad_hosts and
-                        self._last_healing + self.healing_delay < now):
-                    h = self.bad_hosts.pop(0)
-                    self.hosts.append(h)
-                    self._last_healing = now
-
-                if self.shuffle:
-                    random.shuffle(self.hosts)
-
-                hosts = self.hosts + self.bad_hosts
-
-            for h in hosts:
-                bad_host = False
-                try:
-                    self._update_client_host(h)
-                    return orig_func(self._client, *args, **kwargs)
-                except InfluxDBClientError as e:
-                    # Errors caused by user's requests, re-raise
-                    raise e
-                except ValueError as e:
-                    raise e
-                except Exception as e:
-                    # Errors that might caused by server failure, try another
-                    bad_host = True
-                    with self._hosts_lock:
-                        if h in self.hosts:
-                            self.hosts.remove(h)
-                            self.bad_hosts.append(h)
-                        self._last_healing = now
-                finally:
-                    with self._hosts_lock:
-                        if not bad_host and h in self.bad_hosts:
-                            self.bad_hosts.remove(h)
-                            self.hosts.append(h)
-
-            raise InfluxDBServerError("InfluxDB: no viable server!")
-
-        return func
-
-
 def parse_dsn(dsn):
     conn_params = urlparse(dsn)
     init_args = {}
