@@ -2,7 +2,13 @@
 """
 Helper class for InfluxDB
 """
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
 from collections import namedtuple, defaultdict
+from datetime import datetime
 from warnings import warn
 
 import six
@@ -15,6 +21,9 @@ class SeriesHelper(object):
     All data points are immutable, insuring they do not get overwritten.
     Each subclass can write to its own database.
     The time series names can also be based on one or more defined fields.
+    The field "time" can be specified when creating a point, and may be any of
+    the time types supported by the client (i.e. str, datetime, int).
+    If the time is not specified, the current system time (utc) will be used.
 
     Annotated example::
 
@@ -87,8 +96,11 @@ class SeriesHelper(object):
                         ' autocommit is false.'.format(cls.__name__))
 
             cls._datapoints = defaultdict(list)
-            cls._type = namedtuple(cls.__name__, cls._fields + cls._tags)
 
+            if 'time' in cls._fields:
+                cls._fields.remove('time')
+            cls._type = namedtuple(cls.__name__,
+                                   cls._fields + cls._tags + ['time'])
         return super(SeriesHelper, cls).__new__(cls)
 
     def __init__(self, **kw):
@@ -99,6 +111,7 @@ class SeriesHelper(object):
         :warning: Data points are *immutable* (`namedtuples`).
         """
         cls = self.__class__
+        timestamp = kw.pop('time', self._current_timestamp())
 
         if sorted(cls._fields + cls._tags) != sorted(kw.keys()):
             raise NameError(
@@ -106,7 +119,9 @@ class SeriesHelper(object):
                     sorted(cls._fields + cls._tags),
                     kw.keys()))
 
-        cls._datapoints[cls._series_name.format(**kw)].append(cls._type(**kw))
+        cls._datapoints[cls._series_name.format(**kw)].append(
+            cls._type(time=timestamp, **kw)
+        )
 
         if cls._autocommit and \
                 sum(len(series) for series in cls._datapoints.values()) \
@@ -140,6 +155,7 @@ class SeriesHelper(object):
                     "measurement": series_name,
                     "fields": {},
                     "tags": {},
+                    "time": getattr(point, "time")
                 }
 
                 for field in cls._fields:
@@ -157,3 +173,6 @@ class SeriesHelper(object):
         Reset data storage.
         """
         cls._datapoints = defaultdict(list)
+
+    def _current_timestamp(self):
+        return datetime.utcnow()
