@@ -2,6 +2,11 @@
 """
 Helper class for InfluxDB
 """
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
 from collections import namedtuple, defaultdict
 from warnings import warn
 
