@@ -23,7 +23,6 @@ import json
 import requests
 import requests.exceptions
 import socket
-import time
 import requests_mock
 import random
 from nose.tools import raises
@@ -32,8 +31,8 @@ import warnings
 import mock
 import unittest
 
-from influxdb import InfluxDBClient, InfluxDBClusterClient
-from influxdb.client import InfluxDBServerError
+from influxdb import InfluxDBClient
+from influxdb.resultset import ResultSet
 
 
 def _build_response_object(status_code=200, content=""):
@@ -484,39 +483,6 @@ class TestInfluxDBClient(unittest.TestCase):
         with _mocked_session(cli, 'get', 401):
             cli.get_list_database()
 
-    def test_get_list_servers(self):
-        data = {'results': [
-            {'series': [
-                {'columns': ['id', 'cluster_addr', 'raft', 'raft-leader'],
-                 'values': [
-                    [1, 'server01:8088', True, True],
-                    [2, 'server02:8088', True, False],
-                    [3, 'server03:8088', True, False]]}]}
-        ]}
-
-        with _mocked_session(self.cli, 'get', 200, json.dumps(data)):
-            self.assertListEqual(
-                self.cli.get_list_servers(),
-                [{'cluster_addr': 'server01:8088',
-                  'id': 1,
-                  'raft': True,
-                  'raft-leader': True},
-                 {'cluster_addr': 'server02:8088',
-                  'id': 2,
-                  'raft': True,
-                  'raft-leader': False},
-                 {'cluster_addr': 'server03:8088',
-                  'id': 3,
-                  'raft': True,
-                  'raft-leader': False}]
-            )
-
-    @raises(Exception)
-    def test_get_list_servers_fails(self):
-        cli = InfluxDBClient('host', 8086, 'username', 'password')
-        with _mocked_session(cli, 'get', 401):
-            cli.get_list_servers()
-
     def test_create_retention_policy_default(self):
         example_response = '{"results":[{}]}'
 
@@ -626,7 +592,7 @@ class TestInfluxDBClient(unittest.TestCase):
                 text=example_response
             )
             self.assertListEqual(
-                self.cli.get_list_retention_policies(),
+                self.cli.get_list_retention_policies("db"),
                 [{'duration': '24h0m0s',
                   'name': 'fsfdsdf', 'replicaN': 2}]
             )
@@ -679,6 +645,61 @@ class TestInfluxDBClient(unittest.TestCase):
         with self.assertRaises(requests.exceptions.ConnectionError):
             cli.write_points(self.dummy_points)
 
+    @mock.patch('requests.Session.request')
+    def test_random_request_retry(self, mock_request):
+        """Tests that a random number of connection errors will be handled"""
+
+        class CustomMock(object):
+            def __init__(self, retries):
+                self.i = 0
+                self.retries = retries
+
+            def connection_error(self, *args, **kwargs):
+                self.i += 1
+
+                if self.i < self.retries:
+                    raise requests.exceptions.ConnectionError
+                else:
+                    r = requests.Response()
+                    r.status_code = 204
+                    return r
+
+        retries = random.randint(1, 100)
+        mock_request.side_effect = CustomMock(retries).connection_error
+
+        cli = InfluxDBClient(database='db', retries=retries)
+        cli.write_points(
+            self.dummy_points
+        )
+
+    @mock.patch('requests.Session.request')
+    def test_random_request_retry_raises(self, mock_request):
+        """Tests that a random number of connection errors plus one \
+                will not be handled"""
+
+        class CustomMock(object):
+            def __init__(self, retries):
+                self.i = 0
+                self.retries = retries
+
+            def connection_error(self, *args, **kwargs):
+                self.i += 1
+
+                if self.i < self.retries + 1:
+                    raise requests.exceptions.ConnectionError
+                else:
+                    r = requests.Response()
+                    r.status_code = 200
+                    return r
+
+        retries = random.randint(1, 100)
+        mock_request.side_effect = CustomMock(retries).connection_error
+
+        cli = InfluxDBClient(database='db', retries=retries)
+
+        with self.assertRaises(requests.exceptions.ConnectionError):
+            cli.write_points(self.dummy_points)
+
     def test_get_list_users(self):
         example_response = (
             '{"results":[{"series":[{"columns":["user","admin"],'
@@ -723,7 +744,7 @@ class TestInfluxDBClient(unittest.TestCase):
 
             self.assertEqual(
                 m.last_request.qs['q'][0],
-                'grant all privileges to test'
+                'grant all privileges to "test"'
             )
 
     @raises(Exception)
@@ -745,7 +766,7 @@ class TestInfluxDBClient(unittest.TestCase):
 
             self.assertEqual(
                 m.last_request.qs['q'][0],
-                'revoke all privileges from test'
+                'revoke all privileges from "test"'
             )
 
     @raises(Exception)
@@ -767,7 +788,7 @@ class TestInfluxDBClient(unittest.TestCase):
 
             self.assertEqual(
                 m.last_request.qs['q'][0],
-                'grant read on testdb to test'
+                'grant read on "testdb" to "test"'
             )
 
     @raises(Exception)
@@ -789,7 +810,7 @@ class TestInfluxDBClient(unittest.TestCase):
 
             self.assertEqual(
                 m.last_request.qs['q'][0],
-                'revoke read on testdb from test'
+                'revoke read on "testdb" from "test"'
             )
 
     @raises(Exception)
@@ -827,6 +848,43 @@ class TestInfluxDBClient(unittest.TestCase):
         with self.assertRaises(ValueError):
             InfluxDBClient('host', '80/redir', 'username', 'password')
 
+    def test_chunked_response(self):
+        example_response = \
+            u'{"results":[{"statement_id":0,"series":' \
+            '[{"name":"cpu","columns":["fieldKey","fieldType"],"values":' \
+            '[["value","integer"]]}],"partial":true}]}\n{"results":' \
+            '[{"statement_id":0,"series":[{"name":"iops","columns":' \
+            '["fieldKey","fieldType"],"values":[["value","integer"]]}],' \
+            '"partial":true}]}\n{"results":[{"statement_id":0,"series":' \
+            '[{"name":"load","columns":["fieldKey","fieldType"],"values":' \
+            '[["value","integer"]]}],"partial":true}]}\n{"results":' \
+            '[{"statement_id":0,"series":[{"name":"memory","columns":' \
+            '["fieldKey","fieldType"],"values":[["value","integer"]]}]}]}\n'
+
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.GET,
+                "http://localhost:8086/query",
+                text=example_response
+            )
+            response = self.cli.query('show series limit 4 offset 0',
+                                      chunked=True, chunk_size=4)
+            self.assertTrue(len(response) == 4)
+            self.assertEqual(response.__repr__(), ResultSet(
+                {'series': [{'values': [['value', 'integer']],
+                             'name': 'cpu',
+                             'columns': ['fieldKey', 'fieldType']},
+                            {'values': [['value', 'integer']],
+                             'name': 'iops',
+                             'columns': ['fieldKey', 'fieldType']},
+                            {'values': [['value', 'integer']],
+                             'name': 'load',
+                             'columns': ['fieldKey', 'fieldType']},
+                            {'values': [['value', 'integer']],
+                             'name': 'memory',
+                             'columns': ['fieldKey', 'fieldType']}]}
+            ).__repr__())
+
 
 class FakeClient(InfluxDBClient):
 
@@ -835,7 +893,7 @@ class FakeClient(InfluxDBClient):
 
     def query(self,
               query,
-              params={},
+              params=None,
               expected_response_code=200,
               database=None):
         if query == 'Fail':
@@ -846,125 +904,3 @@ class FakeClient(InfluxDBClient):
             raise Exception("Fail Twice")
         else:
             return "Success"
-
-
-class TestInfluxDBClusterClient(unittest.TestCase):
-
-    def setUp(self):
-        # By default, raise exceptions on warnings
-        warnings.simplefilter('error', FutureWarning)
-
-        self.hosts = [('host1', 8086), ('host2', 8086), ('host3', 8086)]
-        self.dsn_string = 'influxdb://uSr:pWd@host1:8086,uSr:pWd@host2:8086/db'
-
-    def test_init(self):
-        cluster = InfluxDBClusterClient(hosts=self.hosts,
-                                        username='username',
-                                        password='password',
-                                        database='database',
-                                        shuffle=False,
-                                        client_base_class=FakeClient)
-        self.assertEqual(3, len(cluster.hosts))
-        self.assertEqual(0, len(cluster.bad_hosts))
-        self.assertIn((cluster._client._host,
-                       cluster._client._port), cluster.hosts)
-
-    def test_one_server_fails(self):
-        cluster = InfluxDBClusterClient(hosts=self.hosts,
-                                        database='database',
-                                        shuffle=False,
-                                        client_base_class=FakeClient)
-        self.assertEqual('Success', cluster.query('Fail once'))
-        self.assertEqual(2, len(cluster.hosts))
-        self.assertEqual(1, len(cluster.bad_hosts))
-
-    def test_two_servers_fail(self):
-        cluster = InfluxDBClusterClient(hosts=self.hosts,
-                                        database='database',
-                                        shuffle=False,
-                                        client_base_class=FakeClient)
-        self.assertEqual('Success', cluster.query('Fail twice'))
-        self.assertEqual(1, len(cluster.hosts))
-        self.assertEqual(2, len(cluster.bad_hosts))
-
-    def test_all_fail(self):
-        cluster = InfluxDBClusterClient(hosts=self.hosts,
-                                        database='database',
-                                        shuffle=True,
-                                        client_base_class=FakeClient)
-        with self.assertRaises(InfluxDBServerError):
-            cluster.query('Fail')
-        self.assertEqual(0, len(cluster.hosts))
-        self.assertEqual(3, len(cluster.bad_hosts))
-
-    def test_all_good(self):
-        cluster = InfluxDBClusterClient(hosts=self.hosts,
-                                        database='database',
-                                        shuffle=True,
-                                        client_base_class=FakeClient)
-        self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(3, len(cluster.hosts))
-        self.assertEqual(0, len(cluster.bad_hosts))
-
-    def test_recovery(self):
-        cluster = InfluxDBClusterClient(hosts=self.hosts,
-                                        database='database',
-                                        shuffle=True,
-                                        client_base_class=FakeClient)
-        with self.assertRaises(InfluxDBServerError):
-            cluster.query('Fail')
-        self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(1, len(cluster.hosts))
-        self.assertEqual(2, len(cluster.bad_hosts))
-
-    def test_healing(self):
-        cluster = InfluxDBClusterClient(hosts=self.hosts,
-                                        database='database',
-                                        shuffle=True,
-                                        healing_delay=1,
-                                        client_base_class=FakeClient)
-        with self.assertRaises(InfluxDBServerError):
-            cluster.query('Fail')
-        self.assertEqual('Success', cluster.query(''))
-        time.sleep(1.1)
-        self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(2, len(cluster.hosts))
-        self.assertEqual(1, len(cluster.bad_hosts))
-        time.sleep(1.1)
-        self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(3, len(cluster.hosts))
-        self.assertEqual(0, len(cluster.bad_hosts))
-
-    def test_dsn(self):
-        cli = InfluxDBClusterClient.from_DSN(self.dsn_string)
-        self.assertEqual([('host1', 8086), ('host2', 8086)], cli.hosts)
-        self.assertEqual('http://host1:8086', cli._client._baseurl)
-        self.assertEqual('uSr', cli._client._username)
-        self.assertEqual('pWd', cli._client._password)
-        self.assertEqual('db', cli._client._database)
-        self.assertFalse(cli._client.use_udp)
-
-        cli = InfluxDBClusterClient.from_DSN('udp+' + self.dsn_string)
-        self.assertTrue(cli._client.use_udp)
-
-        cli = InfluxDBClusterClient.from_DSN('https+' + self.dsn_string)
-        self.assertEqual('https://host1:8086', cli._client._baseurl)
-
-        cli = InfluxDBClusterClient.from_DSN('https+' + self.dsn_string,
-                                             **{'ssl': False})
-        self.assertEqual('http://host1:8086', cli._client._baseurl)
-
-    def test_dsn_password_caps(self):
-        cli = InfluxDBClusterClient.from_DSN(
-            'https+influxdb://usr:pWd@host:8086/db')
-        self.assertEqual('pWd', cli._client._password)
-
-    def test_dsn_mixed_scheme_case(self):
-        cli = InfluxDBClusterClient.from_DSN(
-            'hTTps+inFLUxdb://usr:pWd@host:8086/db')
-        self.assertEqual('pWd', cli._client._password)
-        self.assertEqual('https://host:8086', cli._client._baseurl)
-
-        cli = InfluxDBClusterClient.from_DSN(
-            'uDP+influxdb://usr:pwd@host1:8086,usr:pwd@host2:8086/db')
-        self.assertTrue(cli._client.use_udp)
