@@ -200,14 +200,6 @@ class SimpleTests(SingleTestCaseWithServerMixin,
         self.assertIn('user not found',
                       ctx.exception.content)
 
-    def test_drop_user_invalid(self):
-        with self.assertRaises(InfluxDBClientError) as ctx:
-            self.cli.drop_user('very invalid')
-        self.assertEqual(400, ctx.exception.code)
-        self.assertIn('{"error":"error parsing query: '
-                      'found invalid, expected',
-                      ctx.exception.content)
-
     @unittest.skip("Broken as of 0.9.0")
     def test_revoke_admin_privileges(self):
         self.cli.create_user('test', 'test', admin=True)
@@ -217,13 +209,6 @@ class SimpleTests(SingleTestCaseWithServerMixin,
         self.assertEqual([{'user': 'test', 'admin': False}],
                          self.cli.get_list_users())
 
-    def test_revoke_admin_privileges_invalid(self):
-        with self.assertRaises(InfluxDBClientError) as ctx:
-            self.cli.revoke_admin_privileges('')
-        self.assertEqual(400, ctx.exception.code)
-        self.assertIn('{"error":"error parsing query: ',
-                      ctx.exception.content)
-
     def test_grant_privilege(self):
         self.cli.create_user('test', 'test')
         self.cli.create_database('testdb')
@@ -446,8 +431,8 @@ class CommonTests(ManyTestCasesWithServerMixin,
         rsp = self.cli.get_list_retention_policies()
         self.assertEqual(
             [
-                {'name': 'default',
-                 'duration': '0',
+                {'name': 'autogen',
+                 'duration': '0s',
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
                  'default': True}
@@ -462,11 +447,11 @@ class CommonTests(ManyTestCasesWithServerMixin,
 
         self.assertEqual(
             [
-                {'duration': '0',
+                {'duration': '0s',
                  'default': False,
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
-                 'name': 'default'},
+                 'name': 'autogen'},
                 {'duration': '24h0m0s',
                  'default': True,
                  'replicaN': 1,
@@ -483,14 +468,17 @@ class CommonTests(ManyTestCasesWithServerMixin,
 
     def test_create_retention_policy(self):
         self.cli.create_retention_policy('somename', '1d', 1)
+        # NB: creating a retention policy without specifying
+        # shard group duration
+        #     leads to a shard group duration of 1 hour
         rsp = self.cli.get_list_retention_policies()
         self.assertEqual(
             [
-                {'duration': '0',
+                {'duration': '0s',
                  'default': True,
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
-                 'name': 'default'},
+                 'name': 'autogen'},
                 {'duration': '24h0m0s',
                  'default': False,
                  'replicaN': 1,
@@ -506,18 +494,19 @@ class CommonTests(ManyTestCasesWithServerMixin,
         # Test alter duration
         self.cli.alter_retention_policy('somename', 'db',
                                         duration='4d')
+        # NB: altering retention policy doesn't change shard group duration
         rsp = self.cli.get_list_retention_policies()
         self.assertEqual(
             [
-                {'duration': '0',
+                {'duration': '0s',
                  'default': True,
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
-                 'name': 'default'},
+                 'name': 'autogen'},
                 {'duration': '96h0m0s',
                  'default': False,
                  'replicaN': 1,
-                 'shardGroupDuration': u'24h0m0s',
+                 'shardGroupDuration': u'1h0m0s',
                  'name': 'somename'}
             ],
             rsp
@@ -526,18 +515,19 @@ class CommonTests(ManyTestCasesWithServerMixin,
         # Test alter replication
         self.cli.alter_retention_policy('somename', 'db',
                                         replication=4)
+        # NB: altering retention policy doesn't change shard group duration
         rsp = self.cli.get_list_retention_policies()
         self.assertEqual(
             [
-                {'duration': '0',
+                {'duration': '0s',
                  'default': True,
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
-                 'name': 'default'},
+                 'name': 'autogen'},
                 {'duration': '96h0m0s',
                  'default': False,
                  'replicaN': 4,
-                 'shardGroupDuration': u'24h0m0s',
+                 'shardGroupDuration': u'1h0m0s',
                  'name': 'somename'}
             ],
             rsp
@@ -546,18 +536,19 @@ class CommonTests(ManyTestCasesWithServerMixin,
         # Test alter default
         self.cli.alter_retention_policy('somename', 'db',
                                         default=True)
+        # NB: altering retention policy doesn't change shard group duration
         rsp = self.cli.get_list_retention_policies()
         self.assertEqual(
             [
-                {'duration': '0',
+                {'duration': '0s',
                  'default': False,
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
-                 'name': 'default'},
+                 'name': 'autogen'},
                 {'duration': '96h0m0s',
                  'default': True,
                  'replicaN': 4,
-                 'shardGroupDuration': u'24h0m0s',
+                 'shardGroupDuration': u'1h0m0s',
                  'name': 'somename'}
             ],
             rsp
@@ -573,11 +564,11 @@ class CommonTests(ManyTestCasesWithServerMixin,
         rsp = self.cli.get_list_retention_policies()
         self.assertEqual(
             [
-                {'duration': '0',
+                {'duration': '0s',
                  'default': True,
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
-                 'name': 'default'},
+                 'name': 'autogen'},
                 {'duration': '24h0m0s',
                  'default': False,
                  'replicaN': 1,
@@ -595,11 +586,11 @@ class CommonTests(ManyTestCasesWithServerMixin,
         rsp = self.cli.get_list_retention_policies()
         self.assertEqual(
             [
-                {'duration': '0',
+                {'duration': '0s',
                  'default': True,
                  'replicaN': 1,
                  'shardGroupDuration': u'168h0m0s',
-                 'name': 'default'}
+                 'name': 'autogen'}
             ],
             rsp
         )
