@@ -1,6 +1,9 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import absolute_import
+from __future__ import division
 from __future__ import print_function
+from __future__ import unicode_literals
 
 import datetime
 import os
@@ -10,10 +13,7 @@ import time
 import shutil
 import subprocess
 import sys
-if sys.version_info < (2, 7):
-    import unittest2 as unittest
-else:
-    import unittest
+import unittest
 
 from influxdb.tests.misc import is_port_open, get_free_ports
 
@@ -159,7 +159,7 @@ class InfluxDbInstance(object):
             if not influxdb_bin_path:
                 try:
                     influxdb_bin_path = subprocess.check_output(
-                        ['which', 'influxdb']
+                        ['which', 'influxd']
                     ).strip()
                 except subprocess.CalledProcessError:
                     # fallback on :
